(defvar nntp-article-header-read-size 1024
  "Number of bytes to read when processing headers from NNTP."
)
(defun nnspool-retrieve-headers (sequence)
  "Return list of article headers specified by SEQUENCE of article id.
The format of list is
 `([NUMBER SUBJECT FROM XREF LINES DATE MESSAGE-ID REFERENCES] ...)'.
If there is no References: field, In-Reply-To: field is used instead.
Reader macros for the vector are defined as `nntp-header-FIELD'.
Writer macros for the vector are defined as `nntp-set-header-FIELD'.
Newsgroup must be selected before calling this."
  (save-excursion
	(set-buffer nntp-server-buffer)
	;;(erase-buffer)
	(let ((file nil)
		  (number (length sequence))
		  (count 0)
		  (headers nil)			;Result list.
		  (article 0)
		  (subject nil)
		  (message-id nil)
		  (from nil)
		  (xref nil)
		  (lines 0)
		  (date nil)
		  (references nil))
	  (while sequence
		;;(nntp-send-strings-to-server "HEAD" (car sequence))
		(setq article (car sequence))
		(setq file
			  (concat nnspool-current-directory (prin1-to-string article)))
		(if (and (file-exists-p file)
				 (not (file-directory-p file)))
			(progn
			  (erase-buffer)
			  (insert-file-contents file nil 0 nntp-article-header-read-size)
			  ;; Make message body invisible.
			  (goto-char (point-min))
			  (search-forward "\n\n" nil 'move)
			  (narrow-to-region (point-min) (point))
			  ;; Fold continuation lines.
			  (goto-char (point-min))
			  (while (re-search-forward "\\(\r?\n[ \t]+\\)+" nil t)
				(replace-match " " t t))
			  ;; Make it possible to search for `\nFIELD'.
			  (goto-char (point-min))
			  (insert "\n")
			  ;; Extract From:
			  (goto-char (point-min))
			  (if (search-forward "\nFrom: " nil t)
				  (setq from (buffer-substring
							  (point)
							  (save-excursion (end-of-line) (point))))
				(setq from "(Unknown User)"))
			  ;; Extract Subject:
			  (goto-char (point-min))
			  (if (search-forward "\nSubject: " nil t)
				  (setq subject (buffer-substring
								 (point)
								 (save-excursion (end-of-line) (point))))
				(setq subject "(None)"))
			  ;; Extract Message-ID:
			  (goto-char (point-min))
			  (if (search-forward "\nMessage-ID: " nil t)
				  (setq message-id (buffer-substring
									(point)
									(save-excursion (end-of-line) (point))))
				(setq message-id nil))
			  ;; Extract Date:
			  (goto-char (point-min))
			  (if (search-forward "\nDate: " nil t)
				  (setq date (buffer-substring
							  (point)
							  (save-excursion (end-of-line) (point))))
				(setq date nil))
			  ;; Extract Lines:
			  (goto-char (point-min))
			  (if (search-forward "\nLines: " nil t)
				  (setq lines (string-to-int
							   (buffer-substring
								(point)
								(save-excursion (end-of-line) (point)))))
				(setq lines 0))
			  ;; Extract Xref:
			  (goto-char (point-min))
			  (if (search-forward "\nXref: " nil t)
				  (setq xref (buffer-substring
							  (point)
							  (save-excursion (end-of-line) (point))))
				(setq xref nil))
			  ;; Extract References:
			  ;; If no References: field, use In-Reply-To: field instead.
			  (goto-char (point-min))
			  (if (or (search-forward "\nReferences: " nil t)
					  (search-forward "\nIn-Reply-To: " nil t))
				  (setq references (buffer-substring
									(point)
									(save-excursion (end-of-line) (point))))
				(setq references nil))
			  ;; Collect valid article only.
			  (and article
				   message-id
				   (setq headers
						 (cons (vector article subject from
									   xref lines date
									   message-id references) headers)))
			  ))
		(setq sequence (cdr sequence))
		(setq count (1+ count))
		(and (numberp nntp-large-newsgroup)
			 (> number nntp-large-newsgroup)
			 (zerop (% count 20))
			 (message "NNSPOOL II: Receiving headers... %d%%"
					  (/ (* count 100) number)))
		)
	  (and (numberp nntp-large-newsgroup)
		   (> number nntp-large-newsgroup)
		   (message "NNSPOOL II: Receiving headers... done"))
	  (nreverse headers)
	  )))

(provide 'nnrethr)
